@@ -17,21 +17,24 @@ import os
 from PIL import Image
 
 task_dict = {
-        "pose_body17":"checkpoints/body17.onnx",
-        "pose_body26":"checkpoints/body26.onnx",
-        "pose_wholebody133":"checkpoints/whole133.onnx",
-        "pose_face106":"checkpoints/face106.onnx",
-        "pose_hand21":"checkpoints/hand21.onnx",
-        "det_body":"checkpoints/bodydetect.onnx",
-        "det_coco":"checkpoints/cocodetect.onnx",
-        "det_hand":"checkpoints/handdetect.onnx",
-        "cls_imagenet":"checkpoints/imagenet1k.onnx",
+        "pose_body17":"body17.onnx",
+        "pose_body26":"body26.onnx",
+        "pose_wholebody133":"whole133.onnx",
+        "pose_face106":"face106.onnx",
+        "pose_hand21":"hand21.onnx",
+        "det_body":"bodydetect.onnx",
+        "det_coco":"cocodetect.onnx",
+        "det_hand":"handdetect.onnx",
+        "cls_imagenet":"imagenet1k.onnx",
+        "gen_style":"gen_style_mosaic.onnx",
         "det_face":"...",
         "ocr":"...",
         "mmedu":"...",
         "basenn":"...",
         "baseml":"...",
 }
+style_list = ['mosaic','candy','rain-princess','udnie','pointilism']
+
 class Workflow:
     @classmethod
     def support_task(cls):
@@ -42,19 +45,33 @@ class Workflow:
     @classmethod
     def coco_class(cls):
         return coco_class
+    @classmethod
+    def support_style(cls):
+        return style_list
 
-    def __init__(self, task,checkpoint=None):
+    def __init__(self, task,checkpoint=None,download_path=None,**kwargs):
         task = task.lower()
         self.task_dict = task_dict
         self.task_nick_name ={
             'body':"pose_body17",
+            "face":"pose_face106",
+            "hand":"pose_hand21",
+            "wholebody":"pose_wholebody133",
+            "face106":"pose_face106",
+            "hand21":"pose_hand21",
+            "wholebody133":"pose_wholebody133",
             'body17':"pose_body17",
             'body26':"pose_body26",
             'pose_hand':"pose_hand21",
             'pose_body':"pose_body17",
             "pose_wholebody":"pose_wholebody133",
             "pose_face":"pose_face106",
+            "handdetect":"det_hand",
+            "cocodetect":"det_coco",
+            "facedetect":"det_face",
+            "bodydetect":"det_body",
         }
+        path = download_path if download_path is not None else "checkpoints"
         if task in self.task_nick_name.keys():
             self.task = self.task_nick_name[task]
         else:
@@ -73,11 +90,25 @@ class Workflow:
             'l_blue':[0,0,128],
             'l_purple':[128,0,128],
         }
-        if checkpoint is None and self.task in['det_body','det_coco','pose_body17','pose_body26','pose_wholebody133','pose_face106','pose_hand21','det_hand','cls_imagenet']: # 若不指定权重文件，则使用对应任务的默认模型
-            checkpoint = self.task_dict[self.task]
+        if checkpoint is None and self.task in['det_body','det_coco','pose_body17','pose_body26','pose_wholebody133','pose_face106','pose_hand21','det_hand','cls_imagenet','gen_style']: # 若不指定权重文件，则使用对应任务的默认模型
+            if self.task == 'gen_style':
+                self.style = kwargs.get('style',None)
+                if isinstance(self.style,int): # 按照索引选择风格
+                    style_map = ['mosaic','candy','rain-princess','udnie','pointilism']
+                    self.style = style_map[self.style]
+                elif isinstance(self.style,str) and self.style not in ['mosaic','candy','rain-princess','udnie','pointilism']: # 自定义风格
+                    self.style_img = self.style
+                    self.style = 'custom'
+                if self.style is  None:
+                    self.style = 'mosaic'
+                self.task = 'gen_style_{}'.format(self.style)
+                checkpoint = "gen_style_{}.onnx".format(self.style)
+            else:
+                checkpoint = self.task_dict[self.task]
+            checkpoint = os.path.join(path, checkpoint)
             if not os.path.exists(checkpoint): # 下载默认模型
                 print("本地未检测到{}任务对应模型，云端下载中...".format(self.task))
-                path = "checkpoints"
+                # path = "checkpoints"
                 if not os.path.exists(path):
                     os.mkdir(path)
                 model_name_map = {
@@ -89,9 +120,17 @@ class Workflow:
                     "det_body":"body detection",
                     "det_coco":"coco detection",
                     "det_hand":"hand detection",
+                    "cls_imagenet":"imagenet1k classification",
+                    "gen_style":"mosaic style transfer",
+                    "gen_style_mosaic":"mosaic style transfer",
+                    "gen_style_candy":"candy style transfer",
+                    "gen_style_rain-princess":"rain-princess style transfer",
+                    "gen_style_udnie":"udnie style transfer",
+                    "gen_style_pointilism":"pointilism style transfer",
+                    "gen_style_custom":"custom style transfer",
                 }
                 # download('test12318/bert-english',model_name=model_name,output=path)
-                download(model_repo='XEdu123/xedu.hub_onnx_model', model_name=model_name_map[self.task],output=path)
+                download(model_repo='xedu/hub-model', model_name=model_name_map[self.task],output=path)
             # download()
             self.model = ort.InferenceSession(checkpoint, None)
 
@@ -122,9 +161,12 @@ class Workflow:
                 os.system("pip install scikit-learn")
                 import joblib
             self.model = joblib.load(checkpoint)
+
+
         else:
             self.model = ort.InferenceSession(checkpoint, None)
         print("模型加载成功！")
+    
 
     def inference(self,data=None,show=False,img_type=None,thr=0.3,bbox=None,target_class=None,erase=False,preprocess=None, postprocess=None):
         self._get_img = img_type
@@ -137,6 +179,11 @@ class Workflow:
             return self._face_det_infer(data,show,img_type)
         elif self.task in ['cls_imagenet']:
             return self._cls_infer(data,show, img_type)
+        elif self.task in ['gen_style_mosaic','gen_style_candy','gen_style_rain-princess','gen_style_udnie','gen_style_pointilism','gen_style_custom']:
+            if self.style in ['mosaic','candy','rain-princess','udnie','pointilism'] or isinstance(self.style,int): # 预设风格
+                return self._style_infer(data,show,img_type)
+            else: # 自定义风格
+                return self._style_custom_infer(data,show,img_type)
         elif self.task in ['ocr']:
             return self._ocr_infer(data,show,img_type)
         elif self.task in ['mmedu']:
@@ -161,6 +208,102 @@ class Workflow:
             return self.baseml_res, img
         return self.baseml_res
 
+    def _style_custom_infer(self,data=None,show=None,img_type=None):
+
+        def normalize(image, mean, std):
+            img_data = image.astype(np.float32) / 255
+            mean = np.array(mean, dtype=np.float32)
+            std = np.array(std, dtype=np.float32)
+            norm_img_data = (img_data - mean[:, None, None]) / std[:, None, None]
+            return norm_img_data
+
+        def inv_norm(img_data, mean, std):
+            mean = np.array(mean, dtype=np.float32)
+            std = np.array(std, dtype=np.float32)
+            norm_img_data = img_data * std[:, None, None] + mean[:, None, None]
+            return norm_img_data
+
+        def preprocess(data):
+            # c = cv2.imread(img)
+            if isinstance(data, str):
+                image = cv2.imread(data) 
+            elif isinstance(data, np.ndarray):
+                if len(data.shape) == 2: # grayscale
+                    data = cv2.cvtColor(data, cv2.COLOR_GRAY2BGR) 
+                image = data
+            c = cv2.cvtColor(image, cv2.COLOR_BGR2RGB) 
+            c = np.array(c)
+            c = np.transpose(c,(2, 0, 1))
+            c = normalize(c, [0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
+            c = np.expand_dims(c,0).astype("float32")
+            return c
+
+        def postprocess(out):
+            out = inv_norm(out[0],[0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
+            out = np.transpose(out,(1, 2, 0))
+            out = np.clip(out,0,1)
+            out = (out*255).astype("uint8")
+            return out
+
+
+        c = preprocess(data)
+        s = preprocess(self.style_img)  
+
+        out = self.model.run(None, {"content_img": c, "style_img": s, "alpha":np.array([1.0]).astype("float32")})[0]
+
+        self.style_res = postprocess(out)
+        # print(out.shape)
+        if img_type:
+            self._get_img = img_type
+            if img_type =='cv2':
+                # image = cv2.cvtcolor(image,cv2.BGR2RGB)
+                self.style_res = cv2.cvtColor(self.style_res,cv2.COLOR_BGR2RGB)
+            if show:
+                self.show(self.style_res)
+            return self.style_res,self.style_res
+        return self.style_res
+
+    def _style_infer(self,data=None,show=None,img_type=None):
+        if isinstance(data, str):
+            image = cv2.imread(data) 
+        elif isinstance(data, np.ndarray):
+            if len(data.shape) == 2: # grayscale
+                data = cv2.cvtColor(data, cv2.COLOR_GRAY2BGR) 
+            image = data
+        def preprocess(image):
+            image = cv2.resize(image, (224,224))
+            # Preprocess image
+            x = np.array(image).astype('float32')
+            x = np.transpose(x, [2, 0, 1])
+            x = np.expand_dims(x, axis=0)
+            return x 
+        
+        def postprocess(result,data):
+            result = np.squeeze(result[0])
+            result = np.clip(result, 0, 255)
+            result = result.transpose(1,2,0).astype("uint8")
+            # img = Image.fromarray(result)
+
+            return result
+        data = preprocess(image)
+
+        ort_inputs = {'input1': data}
+        # model = ort.InferenceSession(checkpoint, None)
+        res = self.model.run(['output1'], ort_inputs)
+        # print(self.custom_res[0].shape)
+        self.style_res = postprocess(res,data)
+        if img_type:
+            self._get_img = img_type
+            if img_type =='cv2':
+                # image = cv2.cvtcolor(image,cv2.BGR2RGB)
+                self.style_res = cv2.cvtColor(self.style_res,cv2.COLOR_BGR2RGB)
+            if show:
+                self.show(self.style_res)
+            return self.style_res,self.style_res
+        else:
+            self._get_img = 'pil'
+        return self.style_res
+
 
     def _cls_infer(self,data=None,show=None,img_type=None):
         def preprocess(input_data):
@@ -224,10 +367,14 @@ class Workflow:
 
                 data = np.expand_dims(data,0) # 增加batch维
                 data = data.transpose(0,3,1,2) # （batch,channel,w,h）
-            data  = data.astype(np.float32) # tab iris
+            if isinstance(data,List):
+                data = np.array(data)
+            data  = data.astype(np.float32) 
             ort_inputs = {'input': data}
             # res = ort_session.run(['output'], ort_inputs)
         elif dataset_type == 'tab':
+            if isinstance(data,List):
+                data = np.array(data)
             data  = data.astype(np.float32) # tab iris
             ort_inputs = {'input': data}
             # res = ort_session.run(['output'], ort_inputs)
@@ -241,7 +388,8 @@ class Workflow:
                     self.ix2word = {v:k for k, v in word2idx.items()}
 
                     data = [[word2idx[i] for i in data]]
-                    
+            if isinstance(data,List):
+                data = np.array(data)
             # data  = data.astype(np.float32) # tab iris
         ort_inputs = {'input': data}
         self.basenn_res = ort_session.run(['output'], ort_inputs)
@@ -260,10 +408,12 @@ class Workflow:
 
     def _mmedu_infer(self,data=None,show=False,img_type=None,thr=None):
         if img_type is None:
-            self.result = self.model.inference(data, show=show,get_img=img_type)
+            self.result = self.model.inference(data, show=show,get_img=img_type,score=thr)
+            self.result = self.model.print_result(self.result)
             return self.result
         else:
             self.result,img = self.model.inference(data,get_img=img_type,score=thr)
+            self.result = self.model.print_result(self.result)
             if show ==True:
                 self.show(img)
             return self.result,img
@@ -282,7 +432,7 @@ class Workflow:
             if not os.path.exists(font_file): # 下载默认模型
                 if not os.path.exists(path):
                     os.mkdir(path)
-                download(model_repo='XEdu123/xedu.hub_onnx_model', model_name='ocr font',output=path)
+                download(model_repo='xedu/hub-model', model_name='ocr font',output=path)
             vis = VisRes(font_path=font_file)
             res_img = vis(data, self.bboxs,  self.classes, self.scores)
             if img_type == 'cv2':
@@ -481,6 +631,13 @@ class Workflow:
             img = self.data
         # h,w,c = img.shape
         
+        for color in sketch.keys():
+            # print(color,sketch[color])
+            for [fx,fy] in sketch[color]:
+                # plt.plot([self.keypoints[0][fx][0],self.keypoints[0][fy][0]],[self.keypoints[0][fx][1],self.keypoints[0][fy][1]],color=color)
+                cv2.line(img, (int(self.keypoints[0][fx][0]),int(self.keypoints[0][fx][1])),(int(self.keypoints[0][fy][0]),int(self.keypoints[0][fy][1])),color=self.color_dict[color],thickness=5)
+
+
         for j in range(self.keypoints.shape[0]):
             for i in range(self.keypoints.shape[1]):
                 # plt.scatter(self.keypoints[j][i][0],self.keypoints[j][i][1],c='b',s=10)
@@ -488,15 +645,9 @@ class Workflow:
                 if self.bbox is not None and self.erase:
                     sx1,sy1,sx2,sy2 = self.bbox # 
                     if sx1<x1<sx2 and sy1<y1<sy2 :
-                        cv2.circle(img,(int(self.keypoints[j][i][0]),int(self.keypoints[j][i][1])),radius=1,color=[0,255,0])
+                        cv2.circle(img,(int(self.keypoints[j][i][0]),int(self.keypoints[j][i][1])),radius=2,color=[0,255,0])
                 else:
-                    cv2.circle(img,(int(self.keypoints[j][i][0]),int(self.keypoints[j][i][1])),radius=1,color=[0,255,0])
-
-        for color in sketch.keys():
-            # print(color,sketch[color])
-            for [fx,fy] in sketch[color]:
-                # plt.plot([self.keypoints[0][fx][0],self.keypoints[0][fy][0]],[self.keypoints[0][fx][1],self.keypoints[0][fy][1]],color=color)
-                cv2.line(img, (int(self.keypoints[0][fx][0]),int(self.keypoints[0][fx][1])),(int(self.keypoints[0][fy][0]),int(self.keypoints[0][fy][1])),color=self.color_dict[color])
+                    cv2.circle(img,(int(self.keypoints[j][i][0]),int(self.keypoints[j][i][1])),radius=5,color=[0,255,255],thickness=10)
 
         if self._get_img == 'cv2':
             return img
@@ -505,6 +656,8 @@ class Workflow:
         return img
 
     def show(self,img): # check
+        if isinstance(img,tuple):
+            img = img[1]
         try:
             if self._get_img.lower() == 'cv2':
                 img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
@@ -599,7 +752,7 @@ class Workflow:
             }
             pprint.pprint(formalize_result,sort_dicts=False)
         elif self.task in ['mmedu']:
-            formalize_result = self.model.print_result(self.result)
+            formalize_result = self.result
             pprint.pprint(formalize_result,sort_dicts=False)
         elif self.task in ['basenn']:
             formalize_keys = {
@@ -630,7 +783,6 @@ class Workflow:
                 formalize_keys[lang][0]:self.baseml_res
             }
             pprint.pprint(formalize_result,sort_dicts=False)
-
         elif self.task in ['cls_imagenet']:
             formalize_keys = {
                 "zh":["预测值","分数","预测类别"],
@@ -646,6 +798,8 @@ class Workflow:
                 formalize_keys[language][2]:imagenet_class[res_idx],
             }
             pprint.pprint(formalize_result,sort_dicts=False)
+        elif self.task in ['gen_style_mosaic','gen_style_candy','gen_style_rain-princess','gen_style_udnie','gen_style_pointilism','gen_style_custom']:
+            formalize_result = self.style_res
         return formalize_result
 
 
